
```@meta
CurrentModule = MonteCarloMeasurementUncertainty
DocTestSetup = quote using MonteCarloMeasurementUncertainty end
```

# Example `MCMU` Workflow

## Generating a correlated `datastream`

Consider a `datastream` generated by the following code snippet:

```jldoctest usage
julia> stream_length = Int(2^24)
16777216

julia> datastream = zeros(Float64, stream_length);

julia> for idx in 1:stream_length
           datastream[idx] = cos( π * idx / 8 ) + (idx % 4) * sin( π * idx / (4 + idx ÷ 16) )
       end
```

The function above is crazy and meaningless and just something I made up with correlations. Now we are going to analyze it using both a [`TimeSeries`](@ref) and an [`AccumulatedSeries`](@ref).

## Initialization

One can initialize both [`MonteCarloMeasurement`](@ref)s using the following _pre-allocating_ constructors:

```jldoctest usage
julia> t_series = TimeSeries("Time Series", stream_length);

julia> a_series = AccumulatedSeries("Accumulated Series", stream_length);

```

Note, one does not need to specify the `stream_length`, but then there is no pre-allocated memory. The first argument, a `String` representing the [`name`](@ref) of a given [`MonteCarloMeasurement`](@ref) is optional with the default being empty: `""`. However, this [`name`](@ref) can prove helpful as labels in later analysis or for saving the data. We can retrieve it for an arbitrary [`MonteCarloMeasurement`](@ref) using the [`name`](@ref) function as:

```jldoctest usage
julia> name(t_series) == "Time Series"
true

julia> name(a_series) == "Accumulated Series"
true
```

## [`push!`](@ref)ing data into the [`MonteCarloMeasurement`](@ref)s

At this point, we must input the data from the `datastream` into each measurement. We do this using the [`push!`](@ref) functionality:

```jldoctest usage
julia> push!(t_series, datastream);

julia> push!(a_series, datastream);
```

We note that it's also possible to [`push!`](@ref) single values into [`MonteCarloMeasurement`](@ref)s rather than a whole `Vector` of values.

## Analyzing the `datastream`

```@setup usage
using MonteCarloMeasurementUncertainty
stream_length = Int(2^24)
datastream = zeros(Float64, stream_length)
for idx in 1:stream_length
    datastream[idx] = cos( π * idx / 8 ) + (idx % 4) * sin( π * idx / (4 + idx ÷ 16) )
end
t_series = TimeSeries("Time Series", stream_length)
a_series = AccumulatedSeries("Accumulated Series", stream_length)
push!(t_series, datastream)
push!(a_series, datastream)
```

Finally, we are in a position to analyze the `datastream` collected by either the [`TimeSeries`](@ref) or the [`AccumulatedSeries`](@ref). 

To do so, we can either apply a [`binning_analysis`](@ref) like the following for the [`TimeSeries`](@ref):

```@repl usage
binning_analysis(t_series)
```

Similarly, we can apply it to the [`AccumulatedSeries`](@ref):

```@repl usage
binning_analysis(a_series)
```

Moreover, one can choose to export either [`MonteCarloMeasurement`](@ref) with the [`measurement`](@ref) function extended from the [`Measurements.jl`](https://juliaphysics.github.io/Measurements.jl/stable/) package.

```jldoctest usage
julia> measurement(t_series)
-0.0023 ± 0.0014

julia> measurement(a_series)
-0.0023 ± 0.0014
```

Importantly, this then provides an interface between this [`MonteCarloMeasurementUncertainty.jl`](https://meese-wj.github.io/MonteCarloMeasurementUncertainty.jl/stable) package and the utilities found in the [`Measurements.jl`](https://juliaphysics.github.io/Measurements.jl/stable/) package since the return type of [`measurement`](@ref) is given by

```jldoctest usage
julia> typeof( measurement(t_series) )
Measurements.Measurement{Float64}
```

!!! note
    The [`measurement`](@ref) function automatically calls [`binning_analysis`](@ref) on any [`MonteCarloMeasurement`](@ref) argument.

```@meta
DocTestSetup = nothing
```
